#!/bin/bash
#Description: lib check
#Copyright: Copyright © Huawei Technologies Co., Ltd. 2025. All rights reserved.
#History: 2023-06-19 created

current_dir=$(readlink -f "$(dirname "$0")")
source "${current_dir}/const.sh"
source "${current_dir}/utils.sh"

INSTALL_LOG="${current_dir}/devtools_install.log"
TEMP_INI="${current_dir}/devtools_env_res.ini"

ESSENTIAL_LOST="essential_lost"
NOESSENTIAL_LOST="noessential_lost"

# 环境检查默认结果
ENV_RESULT="${SUCCESS}"

need_install_lib=()
os_lib_list=()
os_cmd_list=()
essential_cmd_list=()
non_essential_cmd_list=()
essential_lib_list=()
non_essential_lib_list=()
lib_path=()


# 版本比较：version_lt [result_version] [standard_version]
function version_lt()
{
    test "$(echo "$@" | tr " " "\n" | sort -rV | head -n 1)" != "$1";
}


function init_variable() {
    # 跟随框架安装的场景下，安装包中的log.sh路径
    local log_sh="${current_dir}/../../../log.sh"
    if [ ! -f "${log_sh}" ]; then
        # 通过节点安装时，安装路径中的log.sh路径
        log_sh="${current_dir}/../../../tools/log.sh"
        if [ ! -f "${log_sh}" ]; then
            # 通过页面安装插件时，插件解压路径的相对路径
            log_sh="${current_dir}/../../../../tools/log.sh"
            if [ ! -f "${log_sh}" ]; then
                echo "There is no log utils(log.sh), please check!"
                exit "${ERROR}"
            else
                source "${log_sh}"
            fi
        else
            source "${log_sh}"
        fi
    else
        source "${log_sh}"
    fi
    if [[ -f "${INSTALL_PATH}/logs/devtools_install.log" ]]; then
        INSTALL_LOG="${INSTALL_PATH}/logs/devtools_install.log"
    fi
    if [[ ! -f "${INSTALL_LOG}" ]]; then
        touch "${INSTALL_LOG}"
        chmod 600 "${INSTALL_LOG}"
    fi
    local config_file="${CONFIG_FILE}"
    if [[ ! -f "${config_file}" ]]; then
        config_file="${current_dir}/../../../install.ini"
    fi
    if [[ ! -f "${config_file}" ]]; then
        echo_and_log "There is no install.ini, please check!" "${ERROR}" "${INSTALL_LOG}"
        exit "${ERROR}"
    fi
    os_type=$(grep -w OS_TYPE "${config_file}" | awk -F= '{if($2 !="") print $2}')
    os_version=$(grep -w OS_VERSION "${config_file}" | awk -F= '{if($2 !="") print $2}')
    package_tool=$(grep -w PACKAGE_TOOL "${config_file}" | awk -F= '{if($2 !="") print $2}')
}


function choose_installer() {
    if [[ -z "${package_tool}" ]]; then
        echo_and_log "Can't found package tool for for this os: ${os_type}" "${ERROR}" "${INSTALL_LOG}"
        for ((i=0;i<${#PACKAGE_TOOLS_LIST[@]};i++))
        do
            if which ${PACKAGE_TOOLS_LIST[$i]} >> /dev/null 2>&1; then
                package_tool=${PACKAGE_TOOLS_LIST[$i]}
            fi
        done
    fi
    echo "package_tool=${package_tool}" >> "${TEMP_INI}"
    echo_and_log "Use ${package_tool} package tool" "${SUCCESS}" "${INSTALL_LOG}"
}

# 检查磁盘空间是否足够安装
function check_install_space() {
    # 磁盘空间大小检测
    echo_and_log "Check disk space" "${DOING}" "${INSTALL_LOG}"
    disk_space_check "${INSTALL_PATH}" "${INSTALL_SPACE_NEED}"
    if [ "$?" -ne "${SUCCESS}" ]; then
        echo "env_disk_lack=${print_disk_space_need}" >> "${TEMP_INI}"
        ENV_RESULT="${ERROR}"
        echo_and_log "Check install path disk space Failed" "${ERROR}" "${INSTALL_LOG}"
    else
        echo_and_log "Check install path disk space Success" "${SUCCESS}" "${INSTALL_LOG}"
    fi
}

function choose_list()
{
    local k=0
    for element in ${package_all[@]}
    do
        if [ "${element}" == "essential" ]; then
            essential_lib_list=(${ESSENTIAL_LIB_LIST["${package_tool}"]})
            for ((k=0;k<${#ESSENTIAL_CMD_LIST[@]};k++))
            do
                essential_cmd_list[${#essential_cmd_list[@]}]=${ESSENTIAL_CMD_LIST[$k]}
            done
        elif [ "${element}" == "non_essential" ]; then
            non_essential_lib_list=(${NON_ESSENTIAL_LIB_LIST["${package_tool}"]})
            if [ "${package_tool}" == "apt" ]; then
                NON_ESSENTIAL_CMD_LIST=(${NON_ESSENTIAL_APT_CMD_LIST[@]})
            else
            	NON_ESSENTIAL_CMD_LIST=(${NON_ESSENTIAL_YUM_CMD_LIST[@]})
            fi
            for ((k=0;k<${#NON_ESSENTIAL_CMD_LIST[@]};k++))
            do
                non_essential_cmd_list[${#non_essential_cmd_list[@]}]=${NON_ESSENTIAL_CMD_LIST[$k]}
            done
        else
            echo_and_log "Install type error: essential or non_essential." "${ERROR}" "${INSTALL_LOG}"
            exit "${ERROR}"
        fi
    done
}

function cmd_exec()
{
    ${os_cmd_list[$1]} >> /dev/null 2>&1
    if [ "$?" -eq "${SUCCESS}" ]; then
        echo_and_log "The ${os_lib_list[$1]}(${os_cmd_list[$1]})  tool cmd check: OK" "${SUCCESS}" "${INSTALL_LOG}"
    else
        echo_and_log "The ${os_lib_list[$1]}(${os_cmd_list[$1]}) tool cmd check: Failed" "${ERROR}" "${INSTALL_LOG}"
        need_install_lib[${#need_install_lib[@]}]=${os_lib_list[$1]}
    fi
}

function package_check()
{
    local package_installed=""
    local dependence_name="$1"
    if [[ "${package_tool}" == "yum" ]] || [[ "${package_tool}" == "zypper" ]]; then
    	if [ "${dependence_name}" = 'java-devel' ]; then
    	    dependence_nick_name='java*-devel'
    	else
    		dependence_nick_name=${dependence_name}
    	fi
        package_installed=$(rpm -qa ${dependence_nick_name})
    else
        package_installed=$(dpkg -l | grep "^ii" | awk '{print $2}' | cut -d ':' -f 1 | grep "^$dependence_name")
    fi
    if [ "${package_installed}" != "" ]; then
        echo_and_log "The $dependence_name tool package check: OK" "${SUCCESS}" "${INSTALL_LOG}"
    else
        # 检测源码安装依赖包
        local lib_name_path=${LIB_CHECK_LIST["$dependence_name"]}
        if [[ -n "${lib_name_path}" ]]; then
            local check_result=""
            get_lib_path
            for lib in ${lib_path[@]}
            do
                local res=$(find ${lib} -name ${lib_name_path} 2>>/dev/null)
                if [[ -n "${res}" ]]; then
                    check_result="0"
                    echo_and_log "The $dependence_name tool package check: OK" "${SUCCESS}" "${INSTALL_LOG}"
                    break
                fi
            done
            if [[ -z "${check_result}" ]]; then
                need_install_lib[${#need_install_lib[@]}]=$dependence_name
            fi
        else
            need_install_lib[${#need_install_lib[@]}]=$dependence_name
        fi
    fi
}

# 获取需要安装依赖包
function cmd_package_check()
{
    local os_lib_list=($1)
    local temp_arr=($2)
    local os_cmd_list=()
    local j=0
    for ((i=0;i<${#temp_arr[@]};i=i+2))
    do
         os_cmd_list[j]="${temp_arr[$i]} ${temp_arr[${i}+1]}"
         let j++
    done
    for ((i=0;i<${#os_lib_list[@]};i++))
    do
        if [ "${i}" -lt "${#os_cmd_list[@]}" ]; then
            # 命令行检测
            cmd_exec "${i}"
        else
            # 包检测
            package_check "${os_lib_list[$i]}"
        fi
    done
    local_need_install_lib=(${need_install_lib[@]})
    need_install_lib=()
}

function lib_check()
{
    echo_and_log "Checking install Dependent Packages" "${DOING}" "${INSTALL_LOG}"
    # 依据os类型进行依赖包检测
    choose_installer
    local i=0
    choose_list

    cmd_package_check "${essential_lib_list[*]}" "${essential_cmd_list[*]}"
    essential_need_install_lib=(${local_need_install_lib[@]})

    cmd_package_check "${non_essential_lib_list[*]}" "${non_essential_cmd_list[*]}"
    non_essential_need_install_lib=(${local_need_install_lib[@]})

    all_need_checked_lib=(${essential_need_install_lib[@]} ${non_essential_need_install_lib[@]})
    essential_need_install_lib_print=(${essential_need_install_lib[@]})

    # 非必需依赖只检查，不安装
    if [ ${#all_need_checked_lib[@]} -eq 0 ]; then
        echo_and_log "lib check success" "${SUCCESS}" "${INSTALL_LOG}"
        exit "${ENV_RESULT}"
    else
        if [ ${#essential_need_install_lib_print[@]} -gt 0 ]; then
            echo_and_log "The following necessary components have not been installed:${essential_need_install_lib_print[*]}" "${DOING}" "${INSTALL_LOG}"
            echo "${ESSENTIAL_LOST}=${essential_need_install_lib_print[*]}" >> "${TEMP_INI}"
        fi
        if [ ${#non_essential_need_install_lib[@]} -gt 0 ]; then
            echo_and_log "The following optional components have not been installed:${non_essential_need_install_lib[*]}" "${DOING}" "${INSTALL_LOG}"
            echo "${NOESSENTIAL_LOST}=${non_essential_need_install_lib[*]}" >> "${TEMP_INI}"
        fi

        #包交互模式
        if [[ "${install_mode}" == "web" ]]; then
            install_essential_fail_array=()
            # 自动安装必需依赖
            if [ ${#essential_need_install_lib_print[@]} -gt 0 ]; then
                local j=0
                for ((i=0;i<${#essential_need_install_lib_print[@]};i++))
                do
                    echo_and_log "${package_tool} install -y ${essential_need_install_lib_print[$i]}" "${DOING}" "${INSTALL_LOG}"
                    ${package_tool} install -y "${essential_need_install_lib_print[$i]}" >> "${INSTALL_LOG}" 2>&1
                    if [ "$?" == "0" ] ; then
                        echo_and_log "${essential_need_install_lib_print[$i]} Successful installation!" "${SUCCESS}" "${INSTALL_LOG}"
                    else
                        echo_and_log "Install ${essential_need_install_lib_print[$i]} failed" "${ERROR}" "${INSTALL_LOG}"
                        install_essential_fail_array[$j]=${essential_need_install_lib_print[$i]}
                        let j++
                    fi
                done
                sed -i '/^essential_lost=/d' "${TEMP_INI}"
                if  [ -n "${install_essential_fail_array}" ]; then
                    echo_and_log "Failed to install the necessary components. Please manually install the following components: ${install_essential_fail_array[*]} " "${ERROR}" "${INSTALL_LOG}"
                    # 更新失败的依赖信息
                    echo "${ESSENTIAL_LOST}=${install_essential_fail_array[*]}" >> "${TEMP_INI}"
                    echo_and_log "Suggestion:" "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "1. Check whether the yum or apt source is correctly configured" "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "2. Manually install the missing components." "${ERROR}" "${INSTALL_LOG}"
                    echo_and_log "Check install Dependent Packages Failed" "${ERROR}" "${INSTALL_LOG}"
                    exit "${ERROR}"
                fi
                echo_and_log "Check install Dependent Packages Success" "${SUCCESS}" "${INSTALL_LOG}"
            fi
        fi
    fi
}


function get_lib_path()
{

    local lib_array=("/lib" "/usr/lib" "/usr/lib64/")
    local lib_path_array=("/etc/profile" )

    for lib_str in ${lib_path_array[@]}
    do
        if [[ -f "${lib_str}" ]]; then
            local lib_path_str=$(grep -w "LD_LIBRARY_PATH=" "${lib_str}" | tail -1 | awk -F= '{if($2 !="") print $2}')
            if [[ -n "${lib_path_str}" ]]; then
                local array=(${lib_path_str//:/ })
                lib_path=(${lib_path[@]} ${array[@]})
            fi
        fi
    done

    if [[ -f "/etc/ld.so.conf" ]]; then
        for line in $(cat "/etc/ld.so.conf")
        do
            lib_path=(${lib_path[@]} ${line})
        done
    fi
    lib_path=(${lib_path[@]} ${lib_array[@]})
}


function main() {
    if [[ ${#*} -gt 2 ]]; then
        echo "Invalid parameters number, must be less or equal two"
        exit "${ERROR}"
    fi
    # 预检查时为包解压路径：/home/devkit_package/DevKit-All-24.0.RC1-Linux-Kunpeng/DevKit-All-24.0.RC1-Linux-Kunpeng/
    # 正式安装时为框架安装路径：/opt/DevKit
    INSTALL_PATH="$1"
    install_mode="$2"
    init_variable
    if [[ "${install_mode}" == "web" ]]; then
        check_install_space
    fi
    package_all=("essential" "non_essential")
    lib_check
    exit "${ENV_RESULT}"
}


main $@
